# %% BcfR0.3-1 R-Source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# __________________________________________________________________________
#
#    bcfr0-bcffish-seq3-nogr-12.R
# __________________________________________________________________________
#
#    Sequential Method 3, with 3-par nonlinear fit:
#    bcf(prolonged uptake phase), alpha(intercept), k2(negative slope in the
#    depuration phase)


# # === start.Fit.CFish.Seq3 =================================================#
# # --- calculates starting values for Seq3 from Seq1 fit
# # -!- silently assumes Data.CFish.Ln and so on; not correct if main
# #     uses different data (??)
# start.Fit.CFish.Seq3 <- function(datacfishln = Data.CFish.Ln,
#                                  testcwater  = Test.CWater.Value,
#                                  timedepur   = Test.TDepur.Value,
#                                  startcalc   = "seq1") {
#                                           # no dispatch yet
#   
#   #-- fit linear depur
#   # $ paralphabeta <- fit.statsLinearLnCFishDepur(datacfishln, timedepur)
#   paralphabeta <- best.Fit.CFish.Depur(datacfishln, timedepur)
#   startalpha = paralphabeta[[1]]
#   # $ startk2    = - paralphabeta[[2]]
#   
#   #-- get k2 and bcf starting values from Fit.BCFPar.CFish.Seq1
#   parfitseq1 <- Fit.BCFPar.CFish.Seq1(datacfishln, testcwater, timedepur)
#   startk1    <- parfitseq1[[1]]
#   startk2    <- parfitseq1[[2]]
#   # startbcf   <- parfitseq1[[3]]
#   
#   #-- nls needs a list
#   startlist <- list(fitk1 = startk1, fitalpha = startalpha, fitk2 = startk2)
#   
#   #-- store as global var
#   Start.Fit.CFish.Seq3 <<- startlist
#   
#   #-- return startlist
#   startlist
#   
# }
# # == start.Fit.CFish.Seq3 =====.


# # == start.Fit.CFish.Seq3.ParBcf ===========================================
# # -- calculates starting values for Seq3 from Seq1 fit
# # ++ silently assumes Data.CFish.Ln and so on; not correct if main
# #     uses different data
# start.Fit.CFish.Seq3.ParBcf <- function(datacfishln = Data.CFish.Ln,
#                                         testcwater  = Test.CWater.Value,
#                                         timedepur   = Test.TDepur.Value,
#                                         startcalc   = "seq1") {
#   # no dispatch yet
#   
#   #- fit linear depur
#   # $ paralphabeta <- fit.statsLinearLnCFishDepur(datacfishln, timedepur)
#   paralphabeta <- best.Fit.CFish.Depur(datacfishln, timedepur)
#   startalpha = paralphabeta[[1]]
#   # $ startk2    = - paralphabeta[[2]]
#   
#   #- get k2 and bcf starting values from Fit.BCFPar.CFish.Seq1
#   parfitseq1 <- Fit.BCFPar.CFish.Seq1(datacfishln, testcwater, timedepur)
#   # startk1    <- parfitseq1[[1]]
#   startk2    <- parfitseq1[[2]]
#   startbcf   <- parfitseq1[[3]]
#   
#   #- nls needs a list
#   startlist <- list(fitbcf = startbcf, fitalpha = startalpha, fitk2 = startk2)
#   
#   #- store as global var
#   Start.Fit.CFish.Seq3.ParBcf <<- startlist
#   
#   #- return startlist
#   startlist
#   
# }
# # == start.Fit.CFish.Seq3.ParBcf =====.


#=== Model.Grid.CFish.Seq3 ================================================:
#--- todo boxcox
Model.Grid.CFish.Seq3 <- function(datacfish = Data.CFish,
                                  testcwater  = Test.CWater.Value,
                                  timedepur   = Test.TDepur.Value,
                                  
                                  k1range, # c(k1min,k1max)
                                  gammarange, # c(gammamin,gammamax)
                                  k2range, # c(k2min,k2max)
                                  
                                  numruns = NULL,
                                  # transform = NULL, only Box-Cox transform
                                  boxcox.power = NULL ) {
                                  # yvaluetoadd   = 0.
  
   
  if (is.null(numruns)) {   
    startdf <- data.frame(runk1 = k1range,
                          rungamma = gammarange,
                          runk2 = k2range)
  } else {
    startdf <- expand.grid(runk1 = seq(k1range[1],k1range[2],len=numruns[1]),
                           rungamma = seq(gammarange[1],gammarange[2],len=numruns[2]),
                           runk2 = seq(k2range[1],k2range[2],len=numruns[3]))
  }
  
  # print(paste("Box-Cox power = ", boxcox.power, sep = ""))
  # print(paste("Y-value added = ", yvaluetoadd, sep =""))
  
  if (is.null(boxcox.power)) {
    #- linear y fit
    modcfishseq3 <- nls2(var.cfish ~ 
                      model.run.cfish.seq3(var.time,
                                           testcwater,
                                           timedepur,
                                           runk1, rungamma, runk2),
                         data  = datacfish,
                         start = startdf,
                         algorithm = "grid-search")
  
    return(modcfishseq3)
  
  } else {
    #- keep yvaluetoadd within function bcPower???
    
    yvaluetoadd = 0.0
    modcfishseq3bcp <- nls2(bcPower(var.cfish + yvaluetoadd, boxcox.power) ~ 
                           bcPower(model.run.cfish.seq3(var.time,
                                                Test.CWater.Value,
                                                Test.TDepur.Value,
                                                runk1, rungamma, runk2) +
                           yvaluetoadd, boxcox.power),
                         data  = datacfish,
                         start = startdf,
                         algorithm = "grid-search")
    
    return(modcfishseq3bcp)
    
  }
}
#~~~ Model.Grid.CFish.Seq3 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.


#=== model.fit.cfish.seq3 =================================================:
#--- fit 3-par nonlinear model on lncfish; params: k1, gamma, k2

model.fit.cfish.seq3 <-   function(datacfish   = Data.CFish, # was: Data.CFish.Ln
                                   testcwater  = Test.CWater.Value,
                                   timedepur   = Test.TDepur.Value,
                                   start       = NULL, # k1, gamma, k2: vector
                                   show.start   = FALSE,
                                   trace       = FALSE,
                                   transformation = "none", # "ln", "boxcox"
                                   boxcox.power = NULL,
                                   # yvaluetoadd = 0.,
                                   # guess       = NULL,
                                   result      = "model") {
  
  optionvec = c("coef", "coef.err","coef.stats", "confint", "covar.mat",
                "deviance", "fit.table", "fitted", "loglik",
                "model", "options", "resid(uals)",
                "resid(ual).stderr", "rss", "summary")
  
  if (tolower(result) == "options")
    return(optionvec)
  
  
#   if (!is.null(guess)) {
#     guessrun <- log(model.run.cfish.seq3(datacfishln$var.time,
#                                          testcwater, timedepur,
#                                          guess[1], guess[2], guess[3]))
#     
#     guessresid <- datacfishln$var.lncfish - guessrun
#     guessndata <- dim(datacfishln)[1]
#     guessrss <- sum(guessresid^2)
#     guessresidstderr <- sqrt(guessrss/(guessndata - 3))
#     guessfittable <- datacfishln
#     guessfittable$var.guessrun <- guessrun
#     guessfittable$var.guessresid <- guessresid
#     
#     # specific result types for guess param?
#     if (tolower(result) == "fit.table") 
#       return(guessfittable)
#     if (tolower(result) == "rss")
#       return(guessrss)
#     if (tolower(result) == "resid.stderr")
#       return(guessresidstderr)
#   }


  #-- if no start is given, construct it from Seq1
  if (is.null(start)) {
    
    #- get stats of k1, gamma, k2, and bcf for Seq1
    #- OR DATA.CFISH ONLY???? !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    fitbcfparseq1 <- Fit.BCFPar.LnCFish.Seq1(datacfishln = Data.CFish.Ln,
                                             testcwater  = Test.CWater.Value,
                                             timedepur   = Test.TDepur.Value)
    
    if (show.start) {
      cat("Self start parameter set from Seq1:\n")
      # noquote(str_trim(lucid(fitbcfparseq1[1:3], dig=5)))
      print(fitbcfparseq1[1:3])
    }
    
    #- build start list
    start <- list(k1 = fitbcfparseq1["k1"],
                  gamma = fitbcfparseq1["gamma"],
                  k2 = fitbcfparseq1["k2"])
          
  } # endif (is.null(start))


  #varcfishtrans <- NULL
  #-- for transformation == "boxcox", boxcox.power must have a value (not NULL)
  if (tolower(transformation) == "boxcox")
    stopifnot(!is.null(boxcox.power))
    
  #-- standard nls model object (result = "model")
  #-- a non-zero yvaluetoadd is not effective for least squares

  modcfishseq3 <- switch(tolower(transformation),
    
    none = nls(var.cfish ~ 
                 model.run.cfish.seq3(var.time,
                                      testcwater,
                                      timedepur,
                                      k1, gamma, k2),
               data  = datacfish, # not Data.CFish.Ln
               start = start,
               trace = trace),
    
    ln   = nls(log(var.cfish) ~ log(model.run.cfish.seq3(var.time,
                                                      testcwater,
                                                      timedepur,
                                                      k1, gamma, k2)),
               data  = datacfish,
               start = start,
               trace = trace),
    
    boxcox = nls(bcPower(var.cfish, boxcox.power) ~ 
                   bcPower(model.run.cfish.seq3(var.time,
                                                testcwater,
                                                timedepur,
                                                k1, gamma, k2), boxcox.power),
                 data  = datacfish, # not Data.CFish.Ln
                 start = start,
                 trace = trace)
    )
  

    
#     if(!yvaluetoadd == 0.)
#       varcfishtrans <- datacfish$var.cfish + yvaluetoadd
     
#   } else {
#     
#     # bcPower from package car; use yvaluetoadd within bcPower?
#     # print(paste("Box-Cox power = ", boxcox.power, sep=""))
#     
#     if (tolower(transformation) == "boxcox") {
#      
#       modcfishseq3 <- nls(bcPower(var.cfish, boxcox.power) ~ 
#                           bcPower(model.run.cfish.seq3(var.time,
#                                                  testcwater,
#                                                  timedepur,
#                                                  k1, gamma, k2), boxcox.power),
#                           data  = datacfish, # not Data.CFish.Ln
#                           start = start,
#                           trace = trace)
#     
#       varcfishtrans <- bcPower(datacfish$var.cfish, boxcox.power)
#     
#     } else {
#       
#       stop("-> Unknown Transformation")
#     }
#   }
  
  #-- default model object
  if(tolower(result) == "model")
    return(modcfishseq3)
  
  #-- best fit coefficients
  if(tolower(result) == "coef")
    return(coef(modcfishseq3))
  
  #-- coefficient statistics
  if (tolower(result) == "coef.stats")
    return(coef(summary(modcfishseq3)))
  
  #-- covariance matrix; main diagonal variances
  if (tolower(result) == "covar.mat")
    return(vcov(modcfishseq3))
  
  #-- deviance, same as RSS 
  if(tolower(result) == "deviance" || tolower(result) == "rss")
  return(deviance(modcfishseq3))
  
  #-- result fitted values if asked
  if (tolower(result) == "fitted")
    return(fitted(modcfishseq3))
  
  #-- result is loglik
  if (tolower(result) == "loglik")
    return(logLik(modcfishseq3))
  
  #-- more extended summary
  if (tolower(result) == "summary")
    return(summary(modcfishseq3))
  
  #-- result fit.table
  if (tolower(result) == "fit.table") {
    datafittable <- Data.CFish
    if(!is.null(varcfishtrans)) {
      datafittable$var.cfish.trans <- varcfishtrans      
    }
    datafittable$var.fitted    <- fitted(modcfishseq3)
    datafittable$var.residuals <- resid(modcfishseq3)
    Data.CFish.Fit <<- datafittable
    return(datafittable)
  }
  
  #-- return the residuals
  if (tolower(result) == "resid" || tolower(result) == "residuals")
    return(resid(modcfishseq3))
  
  #-- return the redisual standard error
  if (tolower(result) == "resid.stderr"||tolower(result)=="residual.stderr") {
    ndat <- dim(datacfishln)[1]
    degfreedom <- ndat - 3
    rss <- deviance(modcfishseq3)
    return(sqrt(rss/degfreedom))
  }
  
  
  #-- to avoid 'Waiting for profiling to be done message' only if necessary
  if(tolower(result) == "confint")
    confintmodcfishseq3 <- confint(modcfishseq3) # matrix
  
  #-- to avoid 'Waiting for profiling to be done message' only if necessary
  if (tolower(result) == "coef.err") {
    
    coefstatsmodcfishseq3 <- coef(summary(modcfishseq3))
    
    coefestimatevec1  <- coefstatsmodcfishseq3[1:3, 1]
    coefstderrorvec1  <- coefstatsmodcfishseq3[1:3, 2]
    
    confintmodcfishseq3 <- confint(modcfishseq3)
    
    coefconfintlovec1 <- confintmodcfishseq3[1:3, 1]
    coefconfintupvec1 <- confintmodcfishseq3[1:3, 2]
        
    coeferrdataframe1 <- data.frame(coefestimatevec1,
                                    coefstderrorvec1,
                                    coefconfintlovec1,
                                    coefconfintupvec1)
    
    names(coeferrdataframe1) <- c("Estimate", "Std.Error","2.5%","97.5%")
    return(coeferrdataframe1)
    
  } #-- endif(tolower(result) == "coef.err")
  
  #-- final dispatch on result
  switch(tolower(result),
         
         #- confidence limits through profiling (result = "confint")
         confint     = confintmodcfishseq3,
         
         stop("Not a valid result option."))   
  
}
#~~~ model.fit.cfish.seq3 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.



#=== Model.Transform.CFish.BoxCox ==============================================:
#--- WORKS ONLY WITH GLOBAL ENVIRONMENT MODEL FIT, NOT VIA Model.Fit.CFish()
Model.Transform.CFish.BoxCox <- function (inputmodelfit, result = "boxcox.model") {
  
  optionvec = c("boxcox.model", "boxcox.summary", "coef.stats.inputmodel",
                "coef.stats.newmodel", "options")
  #--
  if(tolower(result) == "options")
    return(optionvec)
  
  #-- profile log likelihood function
  #-- side effect: plot (modified version Christian Ritz, Jan.2015)
  m2 <- boxcox2.nls(inputmodelfit)
  
  #-- boxcox.model
  if(tolower(result) == "boxcox.model")
    return(m2)
  
  #-- summary values lambda Box-Cox
  if(tolower(result) == "boxcox.summary")
    return(bcSummary(m2))
  
  #-- coef summary boxcox.model
  if(tolower(result) == "coef.stats.newmodel")
    return(coef(summary(m2)))
  
  #-- coef summary(m2))
  if(tolower(result) == "coef.stats.inputmodel")
    return(coef(summary(inputmodelfit)))
}
#~~~ Model.Transform.CFish.BoxCox ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~.



# # == Fit.LnCFish.Seq3 ===================================================
# # -- 
# Fit.LnCFish.Seq3 <- function(datacfishln = Data.CFish.Ln,
#                                 testcwater  = Test.CWater.Value,
#                                 timedepur   = Test.TDepur.Value,
#                                 start       = NULL,
#                                 trace       = FALSE) {
#   
#   #- call the model fitting proc with result = "coef"
#   Model.Fit.LnCFish.Seq3(datacfishln = datacfishln,
#                        testcwater  = testcwater,
#                        timedepur   = timedepur,
#                        start       = start,
#                        trace       = trace,
#                        result      = "coef")
# }
# # == Fit.LnCFish.Seq3 =====.



#=== Fit.BCFPar.LnCFish.Seq3 =================================================
#--- 
Fit.BCFPar.LnCFish.Seq3 <- function (datacfishln = Data.CFish.Ln,
                                   testcwater  = Test.CWater.Value,
                                   timedepur   = Test.TDepur.Value,
                                   start       = NULL,
                                   trace       = FALSE) {
  
  #-- call best.Fit
  bestfit <- Fit.LnCFish.Seq3(datacfishln = datacfishln,
                                 testcwater  = testcwater,
                                 timedepur   = timedepur,
                                 start       = start,
                                 trace       = trace)
  
  #- get fitk1, fitk2 and divide to estimate fitbcf
  bestfitbcf <- bestfit[["fitk1"]]/bestfit[["fitk2"]]
  
  #- concatenate k1, alpha, k2 from bestfit to bestfitbcf
  bestfitvec <- c(bestfit, bestfitbcf)
  
  #- add clean names, without "fit"
  names(bestfitvec) <- c("k1", "alpha", "k2", "bcf")
  
  #- store as global result. NO: only data frame BCFPar.CFish.Seq3
  # Fit.BCFPar.LnCFish.Seq3 <<- bestfitvec
  
  #- return bestfitvec
  bestfitvec
}
#=== Fit.BCFPar.LnCFish.Seq3 =====.



#=== Stats.BCFPar.CFish.Seq3 ==============================================#
#--- if start is used, must be list(fitk1=, fitalpha=, fitk2= )
Stats.BCFPar.CFish.Seq3 <- function (datacfish = Data.CFish,
                                     testcwater  = Test.CWater.Value,
                                     timedepur   = Test.TDepur.Value,
                                     start       = NULL, # k1, gamma, k2: vector
                                     show.start   = FALSE,
                                     trace       = FALSE,
                                     transformation = "none",
                                     boxcox.power = NULL,
                                     # yvaluetoadd = 0.,
                                     guess       = NULL,
                                     digits      = NULL,
                                     result      = "stats") {
  #--
  optionvec <- c("fit","options","stats")
  
  #--
  if(tolower(result)=="options")
    return(optionvec)

  #-- data.frame with rows: k1, gamma, k2 
  #-- columns: Estimate, Std.Error, '2.5%', '97.5%'
  dfk1gammak2 <- model.fit.cfish.seq3(datacfish = datacfish,
                       testcwater  = testcwater,
                       timedepur   = timedepur,
                       start       = start,
                       show.start  = show.start,
                       trace       = trace,
                       transformation = transformation,
                       boxcox.power = boxcox.power,
                       # yvaluetoadd = yvaluetoadd,
                       result = "coef.err")
  
  #-- get covariance matrix
  covarmatk1gammak2 <- model.fit.cfish.seq3(datacfish   = datacfish,
                                      testcwater  = testcwater,
                                      timedepur   = timedepur,
                                      start       = start,
                                      show.start  = show.start,
                                      trace       = trace,
                                      transformation = transformation,
                                      boxcox.power = boxcox.power,
                                      # yvaluetoadd = yvaluetoadd,
                                      result = "covar.mat")
  
#   if(is.null(start)) {
#     start2 <- NULL
#   
#     } else {
#     
#     start2 <- list(fitbcf   = start$fitk1/start$fitk2,
#                    fitalpha = start$fitalpha,
#                    fitk2    = start$fitk2)
#   } # if(is.null(start))
#   
#   #-
#   dfbcfalphak2 <- Model.Fit.LnCFish.Seq3.ParBcf(datacfishln = datacfishln,
#                               testcwater  = testcwater,
#                               timedepur   = timedepur,
#                               start       = start2,
#                               trace       = trace,
#                               result = "coeferr") 
  
  #-- bcfestimate ratio of k1hat and k2hat
  bcfestimate <- dfk1gammak2["k1","Estimate"]/ dfk1gammak2["k2","Estimate"]

  #-- vector of k1, gamma, k2 Estimates (1st component), and bcfestimate
  bcfparest <- c(dfk1gammak2$Estimate, bcfestimate)
  
  #-- Mood & Graybill formula for std of quotient k1/k2
  bcfstderr <- StdErr.BCF.K1K2(k1hat    = dfk1gammak2["k1","Estimate"],
                               k1stderr = dfk1gammak2["k1","Std.Error"],
                               k2hat    = dfk1gammak2["k2","Estimate"],
                               k2stderr = dfk1gammak2["k2","Std.Error"],
                               k1k2cov  = covarmatk1gammak2["k1","k2"])
  
  #-- vector of k1, gamma, k2 stderr, and quotient std error
  bcfparerr <- c(dfk1gammak2$Std.Error, bcfstderr)
  
  #-- vector of k1, gamma, k2 lower limits, and bcf lower limit
  bcfparclo <- c(dfk1gammak2$`2.5%`, bcfestimate - 1.96*bcfstderr)

  #-- vector of k1, gamma, k2 upper limits, and bcf upper limit
  bcfparcup <- c(dfk1gammak2$`97.5%`, bcfestimate + 1.96*bcfstderr)

  #-- compose data.frame from vectors
  dfk1gammak2bcf <- data.frame(bcfparest, bcfparerr, bcfparclo, bcfparcup)
  
  #-- add column names
  names(dfk1gammak2bcf) <- c("Estimate", "Std.Error", "2.5%", "97.5%")
  
  #-- add row names
  row.names(dfk1gammak2bcf) <- c("k1", "gamma", "k2", "bcf")
  
  #-- save global outcome
  Out.Stats.BCFPar.CFish.Seq3 <<- dfk1gammak2bcf

  names(bcfparest) <- c("k1","gamma","k2","bcf")

  if(tolower(result) == "fit") {
    if(is.null(digits)) {
      return(bcfparest)
    } else {
      return(noquote(str_trim(lucid(bcfparest, dig=digits))))
    }
  }
    
  if(tolower(result) == "stats") {
    if(is.null(digits)) {
      return(dfk1gammak2bcf)
    } else {
      return(lucid(dfk1gammak2bcf, dig=digits))
    }
  }
    
}
#=== Stats.BCFPar.CFish.Seq3 =====.


# __________________________________________________________________________
# %% END OF R-SOURCE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%