# bcmf_runaquabc_1.R


RUN_Aqueous_BoxCox <- function(testfile.csvy) {
  
  # title and program version
  cat("\nRUN AQUEOUS (BoxCox-transformed). Version:", PROG.Version, "\n")

  
  # --- READ AQUEOUS TEST DATA

  # read aqueous test data from a .csvy file
  cat("\nAQUEOUS TEST READ: ", testfile.csvy, "\n\n")
  print(readTest_Aqueous(testfile.csvy))

  # show the test conditions
  cat("\nTEST.Aqueous.Param: \n\n")
  print(TEST.Aqueous.Param)

  
  # --- FIT UNTRANSFORMED MODEL FIRST

  cat("\nMODEL FIT (untransformed):\n\n")
  print(fitModel_Aqueous())
  
  
  # --- FIT BCF MODEL BC-TRANSFORMED & DIAGNOSTICS

  # BoxCox profile plot, lambda estimates
  lambdaestimate <- modelTrans_BoxCox_Aqueous()
  
  cat("\nLAMBDA FIT & CONFIDENCE INTERVAL:\n\n")
  print(lambdaestimate)
  
  # take fit as best estimate
  lambdabest <- lambdaestimate["fit"]
  
  # fit BC-transformed
  cat("\nMODEL FIT (BoxCox-transformed):\n\n")
  print(fitModel_Aqueous_BoxCox(lambda = lambdabest))

  # check model diagnostics
  cat("\nMODEL DIAGNOSTICS (BoxCox-transformed):\n")
  print(modelDiag_Aqueous_BoxCox())
  
  
  # --- PLOT BEST FIT BC-TRANSFORMED & ORIGINAL SCALE

  # calculate best fit on BC-transformed scale
  runBestFit_Aqueous_BoxCox()

  # plot best BC-fit on BC-transformed scale
  plotBestFit_Aqueous_BoxCox()
  
  # --- BC-TRANSFORMED CONFIDENCE LIMITS ----------
  
  # Bayesian bootstrap for BC-transformed fit
  bootModel_Aqueous_BoxCox()
  
  # scatter plot bootstrap param
  # print(qplot(BCF.ParamBoot.BoxCox$k2.boot.bc, BCF.ParamBoot.BoxCox$k1.boot.bc))
  
  # print(summary(BCF.ParamBoot.BoxCox$k1.boot.bc))
  # print(summary(BCF.ParamBoot.BoxCox$k2.boot.bc))
  
  # prep lower and upper confidence limits of fit
  runConfFit_Aqueous_BoxCox()
  
  # plot them in BC-transformed space
  plotConfFit_Aqueous_BoxCox()
  
  # plot best BC-fit on original scale
  plotInvBestFit_Aqueous_BoxCox()
 
  # plot on original untransformed scale
  plotInvConfFit_Aqueous_BoxCox()
  
  
  # --- SHOW BCF SUMMARY TABLE (BC-FIT)

  # Test Parameters
  cat("\nAQUEOUS TEST PARAMETERS:\n\n")
  print(TEST.Aqueous.Param)

  # BCF summary Table for BC-transformed fit
  cat("\nBoxCox-TRANSFORMED SUMMARY TABLE:\n\n")
  print(summaryTable_Aqueous_BoxCox(digits = 5))

  cat("\n(End of RUN_Aqueous_BoxCox)\n\n")

}

##############################
