# bcmf_rundietln_3.R


RUN_Dietary_Ln <- function(testfile.csvy) {
  
  # title and program version
  cat("\nRUN DIETARY (Ln-transformed). Version:", PROG.Version, "\n")

  
  # --- READ DIETARY TEST DATA

  # read aqueous test data from a .csvy file
  cat("\nDIETARY TEST READ: ", testfile.csvy, "\n\n")
  print(readTest_Dietary(testfile.csvy))

  # show the test conditions
  cat("\nTEST.Dietary.Param: \n\n")
  print(TEST.Dietary.Param)


  # --- FIT BMF MODEL LN-TRANSFORMED & DIAGNOSTICS

  # fit Ln-transformed
  cat("\nMODEL FIT (Ln-transformed):\n\n")
  print(fitModel_Dietary_Ln())

  # check model diagnostics
  cat("\nMODEL DIAGNOSTICS (Ln-transformed):\n")
  print(modelDiag_Dietary_Ln())
  
  
  # --- PLOT BEST FIT LN-TRANSFORMED & ORIGINAL SCALE

  # calculate best fit on ln-transformed scale
  runBestFit_Dietary_Ln()

  # plot best ln-fit on ln-transformed scale
  plotBestFit_Dietary_Ln()

  # --- LN-TRANSFORMED CONFIDENCE LIMITS ----------
  
  # Bayesian bootstrap for ln-transformed fit
  bootModel_Dietary_Ln()
  
  # scatter plot bootstrap param
  # print(qplot(BMF.ParamBoot.Ln$k2.boot.ln, BMF.ParamBoot.Ln$c0d.boot.ln))
  
  # print(summary(BMF.ParamBoot.Ln$c0d.boot.bc))
  # print(summary(BMF.ParamBoot.Ln$k2.boot.bc))
  
  # prep lower and upper confidence limits of fit
  runConfFit_Dietary_Ln()
  
  # plot them in ln-transformed space
  plotConfFit_Dietary_Ln()  
  
  # plot best ln-fit on original scale
  plotInvBestFit_Dietary_Ln()
 
  # plot on original untransformed scale
  plotInvConfFit_Dietary_Ln()
  
  
  # --- SHOW BMF SUMMARY TABLE (LN-FIT)

  # Test Parameters
  cat("\nDIETARY TEST PARAMETERS:\n\n")
  print(TEST.Dietary.Param)

  # BMF summary Table for Ln-transformed fit
  cat("\nLN-TRANSFORMED SUMMARY TABLE:\n\n")
  print(summaryTable_Dietary_Ln(digits = 5))

  cat("\n(End of RUN_Dietary_Ln)\n\n")

}

##############################
