# %% bcmfR R-source %%
# __________________________________________________________________________
#
#    bcmf_sett_26.R
# __________________________________________________________________________
# %% %%

# // functions to get and set the session information
# // miscellaneous base functions

# _________________

# ==== PROG.Version :

current.PROG.Version <- "0.4-18"

# ==== PROG.Version .


default.NUM.Boot   <- 1000

default.NUM.Digits <- 5

default.NUM.Steps  <- 60


# __________________________________________________________________________

# ==== NUM.Boot --:

# NUM.Boot <- default.NUM.Boot

setNUM.Boot <- function(numboot = default.NUM.Boot) {
  
  assign("NUM.Boot", numboot, envir = .GlobalEnv)
  
  return(numboot)
  
}  #--setNUM.Boot()

# ==== NUM.Boot ==.

# __________________________________________________________________________




#control06 <- nls.control(maxiter = 1000, tol = 1.e-05, minFactor = 1.e-06)


setNUM.Control.nls <- function(maxiter  = 1000,
                               tol       = 1.e-05,
                               minFactor = 1.e-06,
                               printEval = FALSE,
                               warnOnly  = FALSE) {
  
  assign("NUM.Control.nls", nls.control(maxiter = maxiter,
                                        tol = tol,
                                        minFactor = minFactor,
                                        printEval = printEval,
                                        warnOnly  = warnOnly),
         envir = .GlobalEnv)
  
}


# lucid number of digits for SummaryTable
setNUM.Digits <- function(digits = default.NUM.Digits) {
  
  assign("NUM.Digits", digits, envir = .GlobalEnv)
  

}
# set default number of digits


setNUM.Steps.Plot <- function(steps = default.NUM.Steps) {
  
  assign("NUM.Steps.Plot", steps, envir = .GlobalEnv)
  
}



RUN.State   <- 0

USE.GLOBALS <- TRUE

# ??

setTEST.Info <- function(testdata = "", testtype = "") {

  assign("@TEST.Data", testdata, envir = .GlobalEnv)
  
  assign("@TEST.Type", testtype, envir = .GlobalEnv)
  
}


setNUM.Defaults <- function(){
  
  setNUM.Boot()
  
  setNUM.Control.nls()
  
  setNUM.Digits()
  
  setNUM.Steps.Plot()
  
}


InitProgram <- function(progversion = current.PROG.Version) {
  
  assign("PROG.Version", progversion, envir = .GlobalEnv)
  
  setNUM.Defaults()
  
  setTEST.Info()
  
  return(cat(PROG.Version))
  
}



# ====

# graphics elements
# line types
PLOT.Line   <- list(blank = 0, # Murrell(2011).R Graphics, 2nd ed. p.328
                    solid = 1,
                    dashed = 2,
                    dotted = 3,
                    dotdash = 4,
                    dot.dash = 4,
                    longdash = 5,
                    long.dash = 5,
                    twodash = 6,
                    two.dash = 6,
                    #Custom
                    dotted.13 = "13",
                    xlong.dash = "F8",
                    dash.dot.dot = "431313",
                    three.dash = "22848222")

# plotting symbols
PLOT.Symbol <- list(square = 0, #open
                    square.open = 0,
                    circle = 1,
                    circle.open = 1,
                    triangle.up = 2,
                    triangle.up.open = 2,
                    
                    plus = 3,
                    times = 4,
                    
                    diamond = 5,
                    diamond.open = 5,
                    triangle.down = 6,
                    triangle.down.open = 6,
                    
                    square.times = 7,
                    plus.times = 8,
                    star = 8,
                    diamond.plus = 9,
                    circle.plus = 10,
                    hexagram = 11,
                    square.plus = 12,
                    circle.times = 13,
                    square.tick = 14,
                    
                    square.solid = 15, # col = "", etc.
                    circle.solid.medium = 16,
                    triangle.up.solid = 17,
                    diamond.solid.small = 18,
                    circle.solid.large = 19,
                    circle.solid.small = 20,
                    
                    circle.filled = 21, # outline: col = "", fill: bg = "", in legend pt.bg = ""
                    square.filled = 22,
                    diamond.filled = 23,
                    triangle.up.filled = 24,
                    triangle.down.filled = 25,
                    
                    dot = ".",
                    hash = "#",
                    
                    char.A = "A", #65
                    char.B = "B", #66
                    
                    char.a = "a", #97
                    char.b = "b") #98

# ====

PLOT.StyleFit <- list(linebest = list(size  = 1.5,
                                      style = "solid",
                                      color = "darkblue"),
                      lineconf = list(size  = 1.3,
                                      style = "dashed",
                                      color = "darkblue"),
                      symbol   = list(shape = PLOT.Symbol$circle.solid.medium,
                                      size  = 3.0,
                                      color  = "black")) # was "gray50"

PLOT.StyleFit.Ln <- list(linebest = list(size  = 1.5,
                                         style = "solid",
                                         color = "maroon"),
                         lineconf = list(size  = 1.3,
                                         style = "dashed",
                                         color = "maroon"),
                         symbol   = list(shape = PLOT.Symbol$circle.solid.medium,
                                         size  = 3.0,
                                         color = "black"))



PLOT.StyleFit.BoxCox <- list(linebest = list(size  = 1.5,
                                             style = "solid",
                                             color = "darkorange3"),
                             lineconf = list(size  = 1.3,
                                             style = "dashed",
                                             color = "darkorange3"),
                             symbol   = list(shape = PLOT.Symbol$circle.solid.medium,
                                             size  = 3.0,
                                             color = "black"))

PLOT.Title.BestFit <- "Best Fit"
PLOT.Title.BestFit.BoxCox <- "Best Fit (BoxCox-Fitted)"
PLOT.Title.BestFit.Ln <- "Best Fit (Ln-Fitted)"

PLOT.Title.ConfFit <- "Best Fit Confidence"
PLOT.Title.ConfFit.BoxCox <- "Best Fit Confidence (BoxCox-Fitted)"
PLOT.Title.ConfFit.Ln <- "Best Fit Confidence (Ln-Fitted)"

#PLOT.Style.InvConfFit.Ln <- PLOT.Style.ConfFit.Ln


# default graph paper theme ("gray", "base", "bw")
PLOT.Theme.Default <- "base"

# current
PLOT.Theme <- PLOT.Theme.Default

# previous
PLOT.Theme.Last <- PLOT.Theme


getPlotTheme <- function() {
  
  curplottheme <- get("PLOT.Theme", envir = .GlobalEnv)
  
  return(curplottheme)
}

setPlotTheme <- function(theme) {
  
  curplottheme <- get("PLOT.Theme", envir = .GlobalEnv)
  
  assign("PLOT.Theme.Last", curplottheme, envir = .GlobalEnv)
  
  # -- existing theme??
  assign("PLOT.Theme", theme, envir = .GlobalEnv)
  
  return(theme)
}


lastPlotTheme <- function() {
  
  lastplottheme <- get("PLOT.Theme.Last", envir = .GlobalEnv)
  
  assign("PLOT.Theme", lastplottheme, envir = .GlobalEnv)
  
  return(lastplottheme)
}


resetPlotTheme <- function() {
  
  defplottheme <- get("PLOT.Theme.Default", envir = .GlobalEnv)
  
  assign("PLOT.Theme", defplottheme, envir = .GlobalEnv)
  
  return(defplottheme)
}


expand_theme <- function(theme) {

  switch(theme,
    base      = ggthemes::theme_base(),
    bw        = ggplot2::theme_bw(),
    economist = ggthemes::theme_economist(),
    fivethirtyeight = ggthemes::theme_fivethirtyeight(),
    gray      = ggplot2::theme_gray(),
    linedraw  = ggplot2::theme_linedraw(),
    pander    = ggthemes::theme_pander(),
    solarized = ggthemes::theme_solarized(),
    stata     = ggthemes::theme_stata(),
    ggplot2::theme_base())

}



# todo: reset plotid (to 1, if no argument, to given number);
#       increment plotid in plots and keep a list of plots
# PLOT.DEFAULT <- list(plotid      = 1,
#                      linecolor   = "black",
#                      linetype    = PLOT.Line$solid,
#                      linewidth   = 2,
#                      plotsymbol  = PLOT.Symbol$circle,
#                      plottype    = "l") # p.66 in Murrell: "l","p","b","o","h","s"
# 
# PLOT.SETTING <- PLOT.DEFAULT

# __________________________________________________________________________
#
#    bcmf_sett_**.R
# __________________________________________________________________________
# %% END OF R-SCRIPT %%.